<!---
*
* Copyright (C) 2005-2008 Razuna Ltd.
*
* This file is part of Razuna - Enterprise Digital Asset Management.
*
* Razuna is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Razuna is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero Public License for more details.
*
* You should have received a copy of the GNU Affero Public License
* along with Razuna. If not, see <http://www.gnu.org/licenses/>.
*
* You may restribute this Program with a special exception to the terms
* and conditions of version 3.0 of the AGPL as described in Razuna's
* FLOSS exception. You should have received a copy of the FLOSS exception
* along with Razuna. If not, see <http://www.razuna.com/licenses/>.
*
--->
<cfcomponent output="false">
	
	<!--- Setup the DB if DB is not here --->
	<cffunction name="setup" access="public" output="false">
		<cfargument name="thestruct" type="Struct">
		
		<!--- If we come from import then dont do this --->
		<cfif NOT structkeyexists(arguments.thestruct,"fromimport")>
			<!--- Create the DB on the filesystem --->
			<cfinvoke method="BDsetDatasource">
				<cfinvokeargument name="name" value="h2" />
				<cfinvokeargument name="databasename" value="h2" />
				<cfinvokeargument name="logintimeout" value="120" />
				<cfinvokeargument name="initstring" value="" />
				<cfinvokeargument name="connectionretries" value="0" />
				<cfinvokeargument name="connectiontimeout" value="120" />
				<cfinvokeargument name="username" value="razuna" />
				<cfinvokeargument name="password" value="razunadb" />
				<cfinvokeargument name="sqlstoredprocedures" value="true" />
				<cfinvokeargument name="hoststring" value="jdbc:h2:file:#arguments.thestruct.pathoneup#db/razuna;IGNORECASE=TRUE;MODE=Oracle;AUTO_SERVER=TRUE;AUTO_RECONNECT=TRUE;LOG=0;CACHE_SIZE=300000;CACHE_TYPE=SOFT_LRU" />
				<cfinvokeargument name="sqlupdate" value="true" />
				<cfinvokeargument name="sqlselect" value="true" />
				<cfinvokeargument name="sqlinsert" value="true" />
				<cfinvokeargument name="sqldelete" value="true" />
				<cfinvokeargument name="perrequestconnections" value="true" />
				<cfinvokeargument name="drivername" value="org.h2.Driver" />
				<cfinvokeargument name="maxconnections" value="24" />
			</cfinvoke>

			<!---  --->
			<!--- START: CREATE TABLES --->
			<!---  --->
			<!--- CREATE SEQUENCES --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			CREATE SEQUENCE CATEGORIES_SEQ INCREMENT BY 1;
			CREATE SEQUENCE COLLECTION_SEQ INCREMENT BY 1;
			CREATE SEQUENCE CONTENT_ID_SEQ INCREMENT BY 1;
			CREATE SEQUENCE CT_USERS_REMOTEUSERS_SEQ INCREMENT BY 1;
			CREATE SEQUENCE CTUAG_SEQ INCREMENT BY 1;
			CREATE SEQUENCE CTUG_SEQ INCREMENT BY 1;
			CREATE SEQUENCE CTUH_SEQ INCREMENT BY 1;
			CREATE SEQUENCE FILE_SEQ INCREMENT BY 1;
			<!--- CREATE SEQUENCE FLEX_SEQ INCREMENT BY 1; --->
			CREATE SEQUENCE FOLDER_SEQ INCREMENT BY 1 START WITH 3;
			CREATE SEQUENCE GROUPSADMIN_SEQ INCREMENT BY 1;
			CREATE SEQUENCE GROUPS_SEQ START WITH 3 INCREMENT BY 1;
			CREATE SEQUENCE HOSTID_SEQ INCREMENT BY 1;
			CREATE SEQUENCE IMG_SEQ INCREMENT BY 1;
			CREATE SEQUENCE KEYWORDS_SEQ INCREMENT BY 1;
			CREATE SEQUENCE LOG_SEQ INCREMENT BY 1;
			<!--- CREATE SEQUENCE MENUESID_SEQ INCREMENT BY 1 START WITH 5 INCREMENT BY 1; --->
			<!--- CREATE SEQUENCE NEWS_SEQ INCREMENT BY 1; --->
			<!--- CREATE SEQUENCE PAGEID_SEQ INCREMENT BY 1 START WITH 5 INCREMENT BY 1; --->
			<!--- CREATE SEQUENCE PAGEOR_SEQ INCREMENT BY 1; --->
			CREATE SEQUENCE PERMISSIONS_SEQ START WITH 12 INCREMENT BY 1;
			CREATE SEQUENCE PUB_GRP_SEQ INCREMENT BY 1;
			CREATE SEQUENCE PUB_SEQ INCREMENT BY 1;
			CREATE SEQUENCE SCHEDULE_SEQ INCREMENT BY 1;
			CREATE SEQUENCE SCHED_LOG_SEQ INCREMENT BY 1;
			CREATE SEQUENCE USERLOGIN_SEQ INCREMENT BY 1;
			CREATE SEQUENCE USERS_LISTS_SEQ INCREMENT BY 1;
			CREATE SEQUENCE USERS_SEQ START WITH 5 INCREMENT BY 1;
			CREATE SEQUENCE USER_SHIP_SEQ INCREMENT BY 1;
			CREATE SEQUENCE VALUELIST_SEQ INCREMENT BY 1;
			CREATE SEQUENCE CUSTOMFIELD_SEQ INCREMENT BY 1;
			CREATE SEQUENCE TEXT_SEQ INCREMENT BY 1;
			</cfquery>
		</cfif>
		
		<!--- CREATE TABLES --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE MODULES 
		(
			MOD_ID 			BIGINT NOT NULL, 
			MOD_NAME 		VARCHAR(50) NOT NULL, 
			MOD_SHORT 		VARCHAR(3) NOT NULL, 
			MOD_HOST_ID 	BIGINT DEFAULT NULL, 
			CONSTRAINT MODULES_PK PRIMARY KEY (MOD_ID), 
			CONSTRAINT MODULES_UK1 UNIQUE (MOD_NAME, MOD_SHORT, MOD_HOST_ID)
		)
		</cfquery>
		<!--- CREATE PERMISSION --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE PERMISSIONS 
		(
			PER_ID 			BIGINT NOT NULL, 
			PER_KEY  		VARCHAR(50) NOT NULL, 
			PER_HOST_ID 	BIGINT DEFAULT NULL, 
			PER_ACTIVE 		BIGINT DEFAULT 1 NOT NULL, 
			PER_MOD_ID 		BIGINT NOT NULL,
			PER_LEVEL		VARCHAR(10),
			CONSTRAINT PERMISSIONS_PK PRIMARY KEY (PER_ID), 
			CONSTRAINT PERMISSIONS_FK_MODULES FOREIGN KEY (PER_MOD_ID)
			REFERENCES MODULES (MOD_ID)
		)
		</cfquery>
		<!--- CREATE GROUPS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE GROUPS
		(	
			GRP_ID 				BIGINT NOT NULL, 
			GRP_NAME 			VARCHAR(50), 
			GRP_HOST_ID 		BIGINT DEFAULT NULL, 
			GRP_MOD_ID 			BIGINT NOT NULL, 
			GRP_TRANSLATION_KEY VARCHAR(50), 
			CONSTRAINT GROUPS_PK PRIMARY KEY (GRP_ID), 
			CONSTRAINT GROUPS_FK_MODULES FOREIGN KEY (GRP_MOD_ID)
			REFERENCES MODULES (MOD_ID)
		)
		</cfquery>
		<!--- CREATE HOSTS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE HOSTS 
		(
		  HOST_ID           BIGINT NOT NULL,
		  HOST_NAME         VARCHAR(20),
		  HOST_PATH         VARCHAR(50),
		  HOST_CREATE_DATE  DATE,
		  HOST_DB_PREFIX    VARCHAR(40),
		  HOST_LANG         BIGINT,
		  HOST_TYPE			VARCHAR(2) DEFAULT 'F',
		  HOST_SHARD_GROUP	VARCHAR(10),
		  CONSTRAINT HOSTS_PK PRIMARY KEY (HOST_ID)
		)
		</cfquery>
		<!--- CREATE USERS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE USERS 
		(
		  USER_ID              BIGINT NOT NULL,
		  USER_LOGIN_NAME      VARCHAR(50) NOT NULL,
		  USER_EMAIL           VARCHAR(80) NOT NULL,
		  USER_FIRST_NAME      VARCHAR(80),
		  USER_LAST_NAME       VARCHAR(80),
		  USER_PASS            VARCHAR(500) NOT NULL,
		  USER_COMPANY         VARCHAR(80),
		  USER_STREET          VARCHAR(80),
		  USER_STREET_NR       BIGINT(6),
		  USER_STREET_2        VARCHAR(80),
		  USER_STREET_NR_2     BIGINT(6),
		  USER_ZIP             BIGINT(7),
		  USER_CITY            VARCHAR(50),
		  USER_COUNTRY         VARCHAR(60),
		  USER_PHONE           VARCHAR(30),
		  USER_PHONE_2         VARCHAR(30),
		  USER_MOBILE          VARCHAR(30),
		  USER_FAX             VARCHAR(30),
		  USER_CREATE_DATE     DATE,
		  USER_CHANGE_DATE     DATE,
		  USER_ACTIVE          VARCHAR(2),
		  USER_IN_ADMIN        VARCHAR(2),
		  USER_IN_DAM          VARCHAR(2),
		  USER_SALUTATION      VARCHAR(500),
		  USER_IN_VP		   VARCHAR(2) DEFAULT 'F',
		  SET2_NIRVANIX_NAME   VARCHAR(500),
		  SET2_NIRVANIX_PASS   VARCHAR(500),
		CONSTRAINT USERS_PK PRIMARY KEY (USER_ID)
		)
		</cfquery>
		<!--- CREATE CT_GROUPS_USERS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE CT_GROUPS_USERS
		(	
		CT_G_U_GRP_ID 		BIGINT NOT NULL, 
		CT_G_U_USER_ID 		BIGINT NOT NULL, 
		CONSTRAINT CT_GROUPS_USERS_UK1 UNIQUE (CT_G_U_GRP_ID, CT_G_U_USER_ID), 
		CONSTRAINT CT_GROUPS_USERS_GROUPS_FK1 FOREIGN KEY (CT_G_U_GRP_ID)
		REFERENCES GROUPS (GRP_ID)
		)
		</cfquery>
		<!--- CREATE CT_GROUPS_PERMISSIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE CT_GROUPS_PERMISSIONS
		(	
		CT_G_P_PER_ID 		BIGINT NOT NULL, 
		CT_G_P_GRP_ID 		BIGINT NOT NULL, 
		CONSTRAINT CT_GROUPS_PERMISSIONS_UK1 UNIQUE (CT_G_P_PER_ID, CT_G_P_GRP_ID), 
		CONSTRAINT CT_GROUPS_PERMISSIONS_FK2 FOREIGN KEY (CT_G_P_PER_ID)
		REFERENCES PERMISSIONS (PER_ID), 
		CONSTRAINT CT_GROUPS_PERMISSIONS_FK1 FOREIGN KEY (CT_G_P_GRP_ID)
		REFERENCES GROUPS (GRP_ID)
		)
		</cfquery>
		<!--- CREATE LOG_ACTIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE LOG_ACTIONS 
		(
		  LOG_ACT_ID    BIGINT,
		  LOG_ACT_TEXT  VARCHAR(200),
		  CONSTRAINT LOG_ACTIONS_PK PRIMARY KEY (LOG_ACT_ID)
		)
		</cfquery>
		<!--- CREATE CT_USERS_HOSTS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE CT_USERS_HOSTS 
		(
		  CT_U_H_USER_ID  BIGINT,
		  CT_U_H_HOST_ID  BIGINT
		)
		</cfquery>
		<!--- CREATE USERS_LOGIN --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE USERS_LOGIN 
		(
		  USER_LOGIN_ID         BIGINT NOT NULL,
		  USER_ID               BIGINT,
		  USER_LOGIN_DATE       DATE,
		  USER_LOGIN_TIME       TIMESTAMP,
		  USER_LOGIN_PROJECT    BIGINT,
		  USER_LOGIN_SESSION    VARCHAR(200),
		  USER_LOGIN_DATESTAMP  DATE
		)
		</cfquery>
		<!--- CREATE WISDOM --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE WISDOM 
		(
		  WIS_ID      BIGINT,
		  WIS_TEXT    VARCHAR(3000),
		  WIS_AUTHOR  VARCHAR(200),
		  CONSTRAINT WISDOM_PK PRIMARY KEY (WIS_ID)
		)
		</cfquery>
		<!--- CREATE USERS_COMMENTS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE USERS_COMMENTS
		(
		  USER_ID_R           BIGINT,
		  USER_COMMENT        VARCHAR(4000),
		  CREATE_DATE         DATE,
		  CHANGE_DATE         DATE,
		  USER_COMMENT_BY     BIGINT,
		  USER_COMMENT_TITLE  VARCHAR(500),
		  COMMENT_ID          BIGINT
		)
		</cfquery>
		<!--- CREATE FILE_TYPES --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE FILE_TYPES
		(
		  TYPE_ID              VARCHAR(5) CONSTRAINT FILE_TYPE_PK PRIMARY KEY,
		  TYPE_TYPE            VARCHAR(3),
		  TYPE_MIMECONTENT     VARCHAR(50),
		  TYPE_MIMESUBCONTENT  VARCHAR(50)
		)
		</cfquery>
		<!--- CREATE CT_USERS_REMOTEUSERS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE CT_USERS_REMOTEUSERS
		(
		   	CT_U_RU_ID                BIGINT NOT NULL, 
			CT_U_RU_USER_ID           BIGINT NOT NULL, 
			CT_U_RU_REMOTE_URL        VARCHAR(4000) NOT NULL, 
			CT_U_RU_REMOTE_USER_ID    BIGINT NOT NULL, 
			CT_U_RU_REMOTE_USER_NAME  VARCHAR(4000) NOT NULL, 
			CT_U_RU_REMOTE_USER_EMAIL VARCHAR(4000), 
			CT_U_RU_REMOTE_CONFIRMED  BIGINT DEFAULT 0 NOT NULL, 
			CT_U_RU_UUID              VARCHAR(4000) NOT NULL, 
			CT_U_RU_VALIDUNTIL        DATE, 
			CT_U_RU_CONFIRMED         BIGINT DEFAULT 0 NOT NULL, 
		CONSTRAINT CT_USERS_REMOTEUSERS_PK PRIMARY KEY (CT_U_RU_ID),
		CONSTRAINT CT_USERS_REMOTEUSERS_UK1 UNIQUE (CT_U_RU_USER_ID, CT_U_RU_REMOTE_URL, CT_U_RU_REMOTE_USER_ID),
		CONSTRAINT CT_USERS_REMOTEUSERS_UK2 UNIQUE (CT_U_RU_UUID),
		CONSTRAINT CT_USERS_REMOTEUSERS_USER_FK1 FOREIGN KEY (CT_U_RU_USER_ID)
		REFERENCES USERS (USER_ID) ON DELETE CASCADE
		)
		</cfquery>
		<!--- CREATE WEBSERVICES --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE WEBSERVICES
		(
			SESSIONTOKEN 	VARCHAR(100), 
			TIMEOUT 		TIMESTAMP,
			GROUPOFUSER		VARCHAR(20),
			USERID			BIGINT, 
		CONSTRAINT WEBSERVICES_PK PRIMARY KEY (SESSIONTOKEN)
		)
		</cfquery>
		<!--- CREATE SEARCH REINDEX --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE SEARCH_REINDEX
		(
			theid			VARCHAR(100),
			thevalue		INT,
			thehostid		INT,
			datetime		TIMESTAMP
		)
		</cfquery>
		<!---  --->
		<!--- END: CREATE TABLES --->
		<!---  --->
		<cfif NOT structkeyexists(arguments.thestruct,"fromimport")>
			<!---  --->
			<!--- START: INSERT VALUES --->
			<!---  --->
			<!--- USERS --->
			<cfset thepass = hash("admin", "MD5", "UTF-8")>
			<cfquery datasource="#arguments.thestruct.dsn#">
			INSERT INTO USERS
			(USER_ID, USER_LOGIN_NAME, USER_EMAIL, USER_FIRST_NAME, USER_LAST_NAME, USER_PASS, USER_ACTIVE, USER_IN_ADMIN, USER_IN_DAM)
			VALUES (1, 'admin', 'your email here', 'SystemAdmin', 'SystemAdmin', '#thepass#', 'T', 'T', 'T');
			</cfquery>
			<!--- MODULES --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			INSERT INTO modules
			(mod_id, mod_name, mod_short, mod_host_id)
			VALUES(	1, 'razuna', 'ecp', NULL);
			INSERT INTO modules
			(mod_id, mod_name, mod_short, mod_host_id)
			VALUES(	2, 'admin', 'adm', NULL);
			</cfquery>
			<!--- DEFAULT ADMIN GROUPS --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			INSERT INTO groups
			(grp_id, grp_name, grp_host_id, grp_mod_id)
			VALUES(	1, 'SystemAdmin', NULL, 2 );
			INSERT INTO groups
			(grp_id, grp_name, grp_host_id, grp_mod_id)
			VALUES(	2, 'Administrator', NULL, 2	);
			</cfquery>
			<!--- DEFAULT ADMIN CROSS TABLE --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			INSERT INTO ct_groups_users
			(CT_G_U_GRP_ID, CT_G_U_USER_ID)
			VALUES(	1, 1);
			</cfquery>
			<!--- DEFAULT ADMIN PERMISSIONS --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (1,'SystemAdmin',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (2,'Administrator',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (3,'PER_USERS:N',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (4,'PER_USERS:R',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (5,'PER_USERS:W',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (6,'PER_GROUPS:N',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (7,'PER_GROUPS:R',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (8,'PER_GROUPS:W',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (9,'PER_GROUPS_ADMIN:N',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (10,'PER_GROUPS_ADMIN:R',null,1,2,null);
			Insert into PERMISSIONS (PER_ID,PER_KEY,PER_HOST_ID,PER_ACTIVE,PER_MOD_ID,PER_LEVEL) values (11,'PER_GROUPS_ADMIN:W',null,1,2,null);
			</cfquery>
			<!--- DEFAULT ADMIN PERMISSIONS CROSS TABLE --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 1, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 2, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 3, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 4, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 5, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 6, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 7, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 8, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 9, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 10, 1 );
			INSERT INTO	ct_groups_permissions( CT_G_P_PER_ID, CT_G_P_GRP_ID )VALUES( 11, 1 );
			</cfquery>
			<!--- WISDOM --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			2, 'In giving advice, seek to help, not please, your friend.', 'Solon'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			3, 'A friend is one to whom you can pour out the contents of your heart, chaff and grain alike. Knowning that the gentlest of hands will take and sift it, keep what is worth keeping, and with a breath of kindness, blow the rest away.'
			, 'Anonymous'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			4, 'The most exciting phrase to hear in science, the one that heralds new discoveries, is not "Eureka" (I found it!) but "That''s funny ..."'
			, 'Isaac Asimov'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			5, 'Everyone should carefully observe which way his heart draws him, and then choose that way with all his strength!'
			, 'Hasidic saying'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			6, 'Mend your speech a little, lest it may mar your fortunes.', 'Shakespeare, King Lear'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			7, 'In preparing for battle I have always found that plans are useless, but planning is indispensable.'
			, 'Dwight D. Eisenhower'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			8, 'It''s all right to aim high if you have plenty of ammunition.', 'Hawley R. Everhart'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			10, 'A great civilization is not concurred from without until it has destroyed itself from within.'
			, 'Will Durant'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			11, 'Travel far enough away, my friend, and you''ll discover something of great beauty: your self'
			, 'Cirque du Soleil'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			1, 'There are Painters who transform the sun to a yellow spot, but there are others who with the help of their art and their intelligence, transform a yellow spot into the sun.'
			, 'Pablo Picasso'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			9, 'The significant problems we have cannot be solved at the same level of thinking with which we created them.'
			, 'Albert Einstein'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			12, 'Acquaintance, n.: A person whom we know well enough to borrow from, but not well enough to lend to. '
			, 'Ambrose Bierce'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			13, 'The best investment is in the tools of one''s trade.', 'Benjamin Franklin'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			14, 'We all agree that your theory is crazy -- but is it crazy enough?', 'Niels Bohr'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			15, 'Genius without education is like silver in the mine.', 'Benjamin Franklin'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			16, 'Anybody can sympathise with the sufferings of a friend, but it requires a very fine nature to sympathise with a friend''s success.', 'Oscar Wilde'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			17, 'Absurdity, n.: A statement or belief manifestly incosistent with one''s own.', 'Ambrose Bierce'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			18, 'There''s no trick to being a humorist when you have the whole government working for you.', 'Will Rogers');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			19, 'The real question is not whether machines think but whether men do. The mystery which surrounds a thinking machine already surrounds a thinking man.', 'B.F.Skinner'); 
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			20, 'That we must all die, we always knew; I wish I had remembered it sooner.', 'Samuel Johnson');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			21, 'The key to living well is first to will that which is necessary and then to love that which is willed.', 'Irving Yalom');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			22, 'Always tell the truth. You will gratify some people and astonish the rest.', 'Mark Twain');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			23, 'See everything. Ignore a lot. Improve a little.', 'Pope John Paul II');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			24, 'Resentment is like taking poison and hoping the other person dies.', 'St. Augustine');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			25, 'Hope is definitely not the same thing as optimism. It is not the conviction that something will turn out well, but the certainty that something makes sense, regardless of how it turns out.', 'Vaclav Havel');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			26, 'We must never be ashamed of our tears, they are rain from heaven washing the dust from our hard hearts.', 'Charles Dickens');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			27, 'Our business in life is not to succeed, but to continue to fail in good spirits.', 'Robert Louis Stevenson');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			28, 'Be who you are and say what you feel because the people who mind don''t matter and the people who matter don''t mind.', 'Theodor Geisel');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			29, 'It is well to remember that the entire universe, with one trifling exception, is composed of others.', 'John Andrew Holmes');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			30, 'Fail to honor people, they fail to honor you.', 'Lao Tzu');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			31, 'You can leave anything out, as long as you know what it is.', 'Ernest Hemingway');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			32, 'The future is here. It''s just not evenly distributed yet.', 'William Gibson');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			33, 'The future always comes too fast and in the wrong order.', 'Alvin Toffler');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			34, 'There will always be people who are ahead of the curve, and people who are behind the curve. But knowledge moves the curve.', 'Bill James');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			35, 'History is a wave that moves through time slightly faster than we do.', 'Kim Stanley Robinson');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			36, 'Inspiration is for amateurs. I just get to work.', 'Chuck Close');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			37, 'The best and most beautiful things in the world cannot be seen or even touched. They must be felt with the heart.', 'Hellen Keller');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			38, 'Small opportunities are often the beginning of great enterprises.', 'Demosthenes');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			39, 'Simplicity is the utlimate sophistication.', 'Leonardo da Vinci');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			40, 'A journey of thousand miles begins with a single step.', 'Lao tzu');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			41, 'What we think, we become.', 'Buddha');
			INSERT INTO WISDOM ( WIS_ID, WIS_TEXT, WIS_AUTHOR ) VALUES ( 
			42, 'Great minds discuss ideas. Average minds discuss events. Small minds discuss people.', 'Eleanor Roosevelt');
			</cfquery>
			<!--- FILE TYPES --->
			<cfquery datasource="#arguments.thestruct.dsn#">
			INSERT INTO FILE_TYPES VALUES ('doc', 'doc', 'application', 'vnd.ms-word');
			INSERT INTO FILE_TYPES VALUES ('docx', 'doc', 'application', 'vnd.ms-word');
			INSERT INTO FILE_TYPES VALUES ('xls', 'doc', 'application', 'vnd.ms-excel');
			INSERT INTO FILE_TYPES VALUES ('xlsx', 'doc', 'application', 'vnd.ms-excel');
			INSERT INTO FILE_TYPES VALUES ('ppt', 'doc', 'application', 'vnd.ms-powerpoint');
			INSERT INTO FILE_TYPES VALUES ('pptx', 'doc', 'application', 'vnd.ms-powerpoint');
			INSERT INTO FILE_TYPES VALUES ('pdf', 'doc', 'application', 'pdf');
			INSERT INTO FILE_TYPES VALUES ('txt', 'doc', 'application', 'txt');
			INSERT INTO FILE_TYPES VALUES ('psd', 'img', 'application', 'photoshop');
			INSERT INTO FILE_TYPES VALUES ('eps', 'img', 'application', 'eps');
			INSERT INTO FILE_TYPES VALUES ('ai', 'img', 'application', 'photoshop');
			INSERT INTO FILE_TYPES VALUES ('jpg', 'img', 'image', 'jpg');
			INSERT INTO FILE_TYPES VALUES ('jpeg', 'img', 'image', 'jpeg');
			INSERT INTO FILE_TYPES VALUES ('gif', 'img', 'image', 'gif');
			INSERT INTO FILE_TYPES VALUES ('png', 'img', 'image', 'png');
			INSERT INTO FILE_TYPES VALUES ('bmp', 'img', 'image', 'bmp');
			INSERT INTO FILE_TYPES VALUES ('cal', 'img', null, null);
			INSERT INTO FILE_TYPES VALUES ('dcm', 'img', null, null);
			INSERT INTO FILE_TYPES VALUES ('fpx', 'img', 'image', 'vnd.fpx');
			INSERT INTO FILE_TYPES VALUES ('pbm', 'img', 'image', 'pbm');
			INSERT INTO FILE_TYPES VALUES ('pgm', 'img', 'image', 'x-portable-graymap');
			INSERT INTO FILE_TYPES VALUES ('ppm', 'img', 'image', 'x-portable-pixmap');
			INSERT INTO FILE_TYPES VALUES ('pnm', 'img', 'image', 'x-portable-anymap');
			INSERT INTO FILE_TYPES VALUES ('pcx', 'img', 'image', 'pcx');
			INSERT INTO FILE_TYPES VALUES ('pct', 'img', null, null);
			INSERT INTO FILE_TYPES VALUES ('rpx', 'img', null, null);
			INSERT INTO FILE_TYPES VALUES ('ras', 'img', 'image', 'ras');
			INSERT INTO FILE_TYPES VALUES ('tga', 'img', 'image', 'tga');
			INSERT INTO FILE_TYPES VALUES ('tif', 'img', 'image', 'tif');
			INSERT INTO FILE_TYPES VALUES ('tiff', 'img', 'image', 'tiff');
			INSERT INTO FILE_TYPES VALUES ('wbmp', 'img', 'image', 'vnd.wap.wbmp');
			INSERT INTO FILE_TYPES VALUES ('nef', 'img', 'image', 'nef');
			INSERT INTO FILE_TYPES VALUES ('swf', 'vid', 'application', 'x-shockwave-flash');
			INSERT INTO FILE_TYPES VALUES ('flv', 'vid', 'application', 'x-shockwave-flash');
			INSERT INTO FILE_TYPES VALUES ('mov', 'vid', 'video', 'quicktime');
			INSERT INTO FILE_TYPES VALUES ('avi', 'vid', 'video', 'avi');
			INSERT INTO FILE_TYPES VALUES ('3gp', 'vid', 'video', '3gpp');
			INSERT INTO FILE_TYPES VALUES ('rm', 'vid', 'application', 'vnd.rn-realmedia');
			INSERT INTO FILE_TYPES VALUES ('mpg', 'vid', 'video', 'mpeg');
			INSERT INTO FILE_TYPES VALUES ('mp4', 'vid', 'video', 'mp4v-es');
			INSERT INTO FILE_TYPES VALUES ('wmv', 'vid', 'video', 'x-ms-wmv');
			INSERT INTO FILE_TYPES VALUES ('vob', 'vid', 'video', 'mpeg');
			INSERT INTO FILE_TYPES VALUES ('ogv', 'vid', 'video', 'ogv');
			INSERT INTO FILE_TYPES VALUES ('webm', 'vid', 'video', 'webm');
			INSERT INTO FILE_TYPES VALUES ('aff', 'aud', null, null);
			INSERT INTO FILE_TYPES VALUES ('aft', 'aud', null, null);
			INSERT INTO FILE_TYPES VALUES ('au', 'aud', 'audio', 'basic');
			INSERT INTO FILE_TYPES VALUES ('ram', 'aud', 'audio', 'x-pn-realaudio');
			INSERT INTO FILE_TYPES VALUES ('wav', 'aud', 'audio', 'x-wav');
			INSERT INTO FILE_TYPES VALUES ('mp3', 'aud', 'audio', 'mpeg');
			INSERT INTO FILE_TYPES VALUES ('aiff', 'aud', 'audio', 'x-aiff');
			INSERT INTO FILE_TYPES VALUES ('aif', 'aud', 'audio', 'x-aiff');
			INSERT INTO FILE_TYPES VALUES ('aifc', 'aud', 'audio', 'x-aiff');
			INSERT INTO FILE_TYPES VALUES ('wma', 'aud', 'audio', 'x-ms-wma');
			INSERT INTO FILE_TYPES VALUES ('snd', 'aud', 'audio', 'basic');
			INSERT INTO FILE_TYPES VALUES ('mid', 'aud', 'audio', 'mid');
			INSERT INTO FILE_TYPES VALUES ('m3u', 'aud', 'audio', 'x-mpegurl');
			INSERT INTO FILE_TYPES VALUES ('rmi', 'aud', 'audio', 'mid');
			INSERT INTO FILE_TYPES VALUES ('ra', 'aud', 'audio', 'x-pn-realaudio');
			INSERT INTO FILE_TYPES VALUES ('flac', 'aud', 'audio', 'flac');
			INSERT INTO FILE_TYPES VALUES ('ogg', 'aud', 'audio', 'ogg');
			INSERT INTO FILE_TYPES VALUES ('m4a', 'aud', 'audio', 'x-m4a');
			</cfquery>
		</cfif>

		<!--- <cfquery datasource="#arguments.thestruct.dsn#" name="test">
		DROP ALL OBJECTS
		</cfquery> --->
	</cffunction>
	
	<!--- Create Host Remote --->
	<cffunction name="create_host_remote" access="remote" output="false">
		<cfargument name="dsn" type="string" required="true">
		<cfargument name="theschema" type="string" required="true">
		<cfargument name="host_db_prefix" type="string" required="true">
		<!--- Params --->
		<cfset arguments.thestruct = structnew()>
		<cfset arguments.thestruct.dsn = arguments.dsn>
		<cfset arguments.thestruct.theschema = arguments.theschema>
		<cfset arguments.thestruct.host_db_prefix = arguments.host_db_prefix>
		<!--- Create Tables --->
		<cfinvoke method="create_tables" thestruct="#arguments.thestruct#">
		<!--- CREATE THE INDEXES
		<cfinvoke method="create_indexes" thestruct="#arguments.thestruct#"> --->
	</cffunction>
	
	<!--- Create Host --->
	<cffunction name="create_host" access="public" output="false">
		<cfargument name="thestruct" type="Struct">
		<!--- Create Tables --->
		<cfinvoke method="create_tables" thestruct="#arguments.thestruct#">
		<!--- CREATE THE INDEXES
		<cfinvoke method="create_indexes" thestruct="#arguments.thestruct#"> --->
	</cffunction>
	
	<!--- Create Tables --->
	<cffunction name="create_tables" access="public" output="false">
		<cfargument name="thestruct" type="Struct">
		
		<!--- ASSETS_TEMP --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#ASSETS_TEMP 
		(
			TEMPID 			VARCHAR(200), 
			FILENAME 		VARCHAR(500), 
			EXTENSION 		VARCHAR(10), 
			DATE_ADD 		TIMESTAMP, 
			FOLDER_ID		BIGINT, 
			WHO				BIGINT, 
			FILENAMENOEXT	VARCHAR(500), 
			PATH 			VARCHAR(2000), 
			MIMETYPE		VARCHAR(200), 
			THESIZE			BIGINT,
			GROUPID			BIGINT,
			SCHED_ACTION	INT,
			SCHED_ID		INT,
			FILE_ID			INT,
			LINK_KIND		VARCHAR(20),
			HOST_ID			BIGINT
		)
		</cfquery>
		
		<!--- XMP --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#XMP 
		(
			id_r					bigint,
			asset_type				varchar(10),
			subjectcode				varchar(200),
			creator					varchar(200),
			title					varchar(200),
			authorsposition			varchar(200),
			captionwriter			varchar(200),
			ciadrextadr				varchar(200),
			category				varchar(200),
			supplementalcategories	VARCHAR(4000),
			urgency					varchar(200),
			description				VARCHAR(4000),
			ciadrcity				varchar(200),
			ciadrctry				varchar(200),
			location				varchar(200),
			ciadrpcode				varchar(200),
			ciemailwork				varchar(200),
			ciurlwork				varchar(200),
			citelwork				varchar(200),
			intellectualgenre		varchar(200),
			instructions			VARCHAR(4000),
			source					varchar(200),
			usageterms				VARCHAR(4000),
			copyrightstatus			VARCHAR(4000),
			transmissionreference	varchar(200),
			webstatement			varchar(500),
			headline				varchar(200),
			datecreated				varchar(200),
			city					varchar(200),
			ciadrregion				varchar(200),
			country					varchar(200),
			countrycode				varchar(200),
			scene					varchar(200),
			state					varchar(200),
			credit					varchar(200),
			rights					VARCHAR(4000),
			HOST_ID					BIGINT
		)  
		</cfquery>
		
		<!--- CART --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#CART
		(
		  CART_ID           	VARCHAR(200),
		  USER_ID           	BIGINT,
		  CART_QUANTITY     	BIGINT,
		  CART_PRODUCT_ID   	BIGINT,
		  CART_CREATE_DATE  	DATE,
		  CART_CREATE_TIME  	TIMESTAMP,
		  CART_CHANGE_DATE  	DATE,
		  CART_CHANGE_TIME  	TIMESTAMP,
		  CART_FILE_TYPE    	VARCHAR(5),
		  cart_order_email 		varchar(150),
		  cart_order_message 	varchar(2000),
		  cart_order_done 		varchar(1), 
		  cart_order_date 		timestamp,
		  cart_order_user_r 	BIGINT,
		  HOST_ID				BIGINT
		)
		</cfquery>
				
		<!--- FOLDERS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#FOLDERS
		(
		  FOLDER_ID             BIGINT,
		  FOLDER_NAME           VARCHAR(200),
		  FOLDER_LEVEL          BIGINT,
		  FOLDER_ID_R           BIGINT DEFAULT NULL,
		  FOLDER_MAIN_ID_R      BIGINT,
		  FOLDER_OWNER          BIGINT,
		  FOLDER_CREATE_DATE    DATE,
		  FOLDER_CREATE_TIME    TIMESTAMP,
		  FOLDER_CHANGE_DATE    DATE,
		  FOLDER_CHANGE_TIME    TIMESTAMP,
		  FOLDER_IS_IMG_FOLDER  VARCHAR(2),
		  FOLDER_IMG_PUB_ID     BIGINT,
		  FOLDER_OF_USER        VARCHAR(2) DEFAULT NULL,
		  FOLDER_IS_COLLECTION  VARCHAR(2) DEFAULT NULL,
		  FOLDER_IS_VID_FOLDER  VARCHAR(2),
		  FOLDER_VID_PUB_ID		BIGINT,
		  FOLDER_AVAILABLE_DSC  BIGINT DEFAULT 1,
		  FOLDER_SHARED			VARCHAR(2) DEFAULT 'F',
		  FOLDER_NAME_SHARED	VARCHAR(200),
		  LINK_PATH				VARCHAR(200),
		  share_dl_org			varchar(1) DEFAULT 'f',
     	  share_comments		varchar(1) DEFAULT 'f',
		  share_upload			varchar(1) DEFAULT 'f',
		  share_order			varchar(1) DEFAULT 'f',
		  share_order_user		BIGINT,
		  HOST_ID				BIGINT,
		  CONSTRAINT #arguments.thestruct.host_db_prefix#FOLDERS_HOSTID_FK1 FOREIGN KEY (HOST_ID)
		  REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- FOLDERS DESC --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#FOLDERS_DESC
		(
		  FOLDER_ID_R		BIGINT,
		  LANG_ID_R			BIGINT,
		  FOLDER_DESC		VARCHAR(1000),
		  HOST_ID			BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#FOLDER_DESC_FK_HOSTID FOREIGN KEY (HOST_ID)
		REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- FOLDERS GROUPS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#FOLDERS_GROUPS
		(
		  FOLDER_ID_R		BIGINT,
		  GRP_ID_R			BIGINT DEFAULT NULL,
		  GRP_PERMISSION	VARCHAR(2),
		  HOST_ID			BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#FOLDER_GROUPS_FK_HOSTID FOREIGN KEY (HOST_ID)
		REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- FILES --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#FILES
		(
		  FILE_ID              BIGINT,
		  FOLDER_ID_R          BIGINT DEFAULT NULL,
		  FILE_CREATE_DATE     DATE,
		  FILE_CREATE_TIME     TIMESTAMP,
		  FILE_CHANGE_DATE     DATE,
		  FILE_CHANGE_TIME     TIMESTAMP,
		  FILE_OWNER           BIGINT,
		  FILE_TYPE            VARCHAR(5),
		  FILE_NAME            VARCHAR(200),
		  FILE_EXTENSION       VARCHAR(10) DEFAULT NULL,
		  FILE_NAME_NOEXT      VARCHAR(200),
		  FILE_CONTENTTYPE     VARCHAR(100),
		  FILE_CONTENTSUBTYPE  VARCHAR(100),
		  FILE_REMARKS         VARCHAR(4000),
		  FILE_ONLINE          VARCHAR(2),
		  FILE_NAME_ORG        VARCHAR(200),
		  FILE_SIZE			   BIGINT,
		  LUCENE_KEY		   VARCHAR(2000),
		  SHARED			   VARCHAR(2) DEFAULT 'F',
		  LINK_KIND			   VARCHAR(20),
		  LINK_PATH_URL		   VARCHAR(2000),
		  FILE_META			   CLOB,
		  HOST_ID			   BIGINT,
		  PATH_TO_ASSET		   VARCHAR(100),
		  CLOUD_URL			   VARCHAR(500),
		  CLOUD_URL_ORG		   VARCHAR(500),
		  HASHTAG			   VARCHAR(100),
		  IS_AVAILABLE		  VARCHAR(1) DEFAULT 0,
		  CLOUD_URL_EXP		   BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#FILE_PK PRIMARY KEY (FILE_ID),
		CONSTRAINT #arguments.thestruct.host_db_prefix#FILE_FK_HOST FOREIGN KEY (HOST_ID)
		REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- FILES DESC --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#FILES_DESC
		(
		  FILE_ID_R      BIGINT,
		  LANG_ID_R      BIGINT,
		  FILE_DESC      VARCHAR(1000),
		  FILE_KEYWORDS  VARCHAR(2000),
		  ID			 BIGINT AUTO_INCREMENT,
		  HOST_ID		 BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#FILE_DESC_FK_FILE FOREIGN KEY (FILE_ID_R)
		REFERENCES #arguments.thestruct.host_db_prefix#FILES (FILE_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- IMAGES --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#IMAGES
		(
		  IMG_ID              BIGINT,
		  METABLOB        	  VARCHAR(1),
		  METAEXIF            VARCHAR(1),
		  METAIPTC            VARCHAR(1),
		  METAXMP             VARCHAR(1),
		  IMAGE               VARCHAR(1),
		  THUMB               VARCHAR(1),
		  COMP                VARCHAR(1),
		  COMP_UW             VARCHAR(1),
		  IMG_GROUP           BIGINT DEFAULT NULL,
		  IMG_PUBLISHER       VARCHAR(200),
		  IMG_FILENAME        VARCHAR(200),
		  FOLDER_ID_R         BIGINT DEFAULT NULL,
		  IMG_CUSTOM_ID       VARCHAR(100),
		  IMG_ONLINE          VARCHAR(2),
		  IMG_OWNER           BIGINT,
		  IMG_CREATE_DATE     DATE,
		  IMG_CREATE_TIME     TIMESTAMP,
		  IMG_CHANGE_DATE     DATE,
		  IMG_CHANGE_TIME     TIMESTAMP,
		  IMG_RANKING         BIGINT,
		  IMG_SINGLE_SALE     VARCHAR(2),
		  IMG_IS_NEW          VARCHAR(2),
		  IMG_SELECTION       VARCHAR(2),
		  IMG_IN_PROGRESS     VARCHAR(2),
		  IMG_ALIGNMENT       VARCHAR(200),
		  IMG_LICENSE         VARCHAR(200),
		  IMG_DOMINANT_COLOR  VARCHAR(200),
		  IMG_COLOR_MODE      VARCHAR(200),
		  IMG_IMAGE_TYPE      VARCHAR(200),
		  IMG_CATEGORY_ONE    VARCHAR(2000),
		  IMG_REMARKS         VARCHAR(1),
		  IMG_EXTENSION       VARCHAR(6),
		  THUMB_EXTENSION	  VARCHAR(6),
		  THUMB_WIDTH         BIGINT,
		  THUMB_HEIGHT        BIGINT,
		  IMG_FILENAME_ORG    VARCHAR(200),
		  IMG_WIDTH           BIGINT,
  		  IMG_HEIGHT          BIGINT,
	 	  IMG_SIZE            BIGINT,
  		  THUMB_SIZE          BIGINT,
		  LUCENE_KEY		  VARCHAR(2000),
		  SHARED			  VARCHAR(2) DEFAULT 'F',
		  LINK_KIND			  VARCHAR(20),
		  LINK_PATH_URL		  VARCHAR(2000),
		  IMG_META			  CLOB,
		  HOST_ID			  BIGINT,
		  PATH_TO_ASSET		  VARCHAR(100),
		  CLOUD_URL			   VARCHAR(500),
		  CLOUD_URL_ORG		   VARCHAR(500),
		  HASHTAG			   VARCHAR(100),
		  IS_AVAILABLE		 VARCHAR(1) DEFAULT 0,
		  CLOUD_URL_EXP		   BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#IMAGE_PK PRIMARY KEY (IMG_ID),
		CONSTRAINT #arguments.thestruct.host_db_prefix#IMAGE_FK_HOSTID FOREIGN KEY (HOST_ID)
		REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- IMAGES TEXT --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#IMAGES_TEXT
		(
		  IMG_ID_R         BIGINT NOT NULL,
		  LANG_ID_R        BIGINT NOT NULL,
		  IMG_KEYWORDS     VARCHAR(4000),
		  IMG_DESCRIPTION  VARCHAR(4000),
		  ID			   BIGINT AUTO_INCREMENT,
		  HOST_ID		   BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#IMAGE_TEXT_FK_IMG FOREIGN KEY (IMG_ID_R)
		REFERENCES #arguments.thestruct.host_db_prefix#IMAGES (IMG_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- LOG ASSETS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#LOG_ASSETS
		(
		  LOG_ID			BIGINT NOT NULL, 
		  LOG_USER			BIGINT, 
		  LOG_ACTION		VARCHAR(100), 
		  LOG_DATE			DATE, 
		  LOG_TIME			TIMESTAMP, 
		  LOG_DESC			VARCHAR(4000), 
		  LOG_FILE_TYPE		VARCHAR(5), 
		  LOG_BROWSER		VARCHAR(500), 
		  LOG_IP			VARCHAR(200), 
		  LOG_TIMESTAMP		TIMESTAMP,
		  HOST_ID			BIGINT,
		  CONSTRAINT #arguments.thestruct.host_db_prefix#LOG_ASSETS_PK PRIMARY KEY (LOG_ID)
		)
		</cfquery>
		
		<!--- LOG FOLDERS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#LOG_FOLDERS
		(
		  LOG_ID			BIGINT NOT NULL, 
		  LOG_USER			BIGINT, 
		  LOG_ACTION		VARCHAR(100), 
		  LOG_DATE			DATE, 
		  LOG_TIME			TIMESTAMP, 
		  LOG_DESC			VARCHAR(4000), 
		  LOG_BROWSER		VARCHAR(500), 
		  LOG_IP			VARCHAR(200), 
		  LOG_TIMESTAMP		TIMESTAMP,
		  HOST_ID			BIGINT, 
		  CONSTRAINT #arguments.thestruct.host_db_prefix#LOG_FOLDERS_PK PRIMARY KEY (LOG_ID)
		)
		</cfquery>
		
		<!--- LOG USERS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#LOG_USERS
		(
		  LOG_ID			BIGINT NOT NULL, 
		  LOG_USER			BIGINT, 
		  LOG_ACTION		VARCHAR(100), 
		  LOG_DATE			DATE, 
		  LOG_TIME			TIMESTAMP, 
		  LOG_DESC			VARCHAR(4000), 
		  LOG_BROWSER		VARCHAR(500), 
		  LOG_IP			VARCHAR(200), 
		  LOG_TIMESTAMP		TIMESTAMP,
		  LOG_SECTION		VARCHAR(10),
		  HOST_ID			BIGINT,
		  CONSTRAINT #arguments.thestruct.host_db_prefix#LOG_USERS_PK PRIMARY KEY (LOG_ID)
		)
		</cfquery>
		
		<!--- LOG SEARCH --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#LOG_SEARCH
		(
		  LOG_ID			BIGINT NOT NULL,
		  LOG_USER			BIGINT,
		  LOG_DATE			DATE,
		  LOG_TIME			TIMESTAMP,
		  LOG_SEARCH_FOR	VARCHAR(500),
		  LOG_FOUNDITEMS	BIGINT,
		  LOG_SEARCH_FROM	VARCHAR(50),
		  LOG_TIMESTAMP		TIMESTAMP,
		  LOG_BROWSER		VARCHAR(500), 
		  LOG_IP			VARCHAR(200),
		  HOST_ID			BIGINT, 
		CONSTRAINT #arguments.thestruct.host_db_prefix#LOG_SEARCH_PK PRIMARY KEY (LOG_ID)
		)
		</cfquery>
				
		<!--- SETTINGS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#SETTINGS
		(
		  SET_ID			VARCHAR(100) NOT NULL,
		  SET_PREF			VARCHAR(2000),
		  HOST_ID			BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#SETTINGS_FK FOREIGN KEY (HOST_ID)
		REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- SETTINGS 2 --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#SETTINGS_2
		(
		  SET2_ID                       BIGINT NOT NULL,
		  SET2_DATE_FORMAT              VARCHAR(20),
		  SET2_DATE_FORMAT_DEL          VARCHAR(3),
		  SET2_META_KEYWORDS            VARCHAR(260),
		  SET2_META_DESC                VARCHAR(500),
		  SET2_META_AUTHOR              VARCHAR(200),
		  SET2_META_PUBLISHER           VARCHAR(200),
		  SET2_META_COPYRIGHT           VARCHAR(200),
		  SET2_META_ROBOTS              VARCHAR(200),
		  SET2_META_REVISIT             VARCHAR(200),
		  SET2_META_CUSTOM              VARCHAR(1000),
		  SET2_URL_SP_ORIGINAL          VARCHAR(400),
		  SET2_URL_SP_THUMB             VARCHAR(400),
		  SET2_URL_SP_COMP              VARCHAR(400),
		  SET2_URL_SP_COMP_UW           VARCHAR(400),
		  SET2_INTRANET_LOGO            VARCHAR(1),
		  SET2_URL_APP_SERVER           VARCHAR(400),
		  SET2_ORA_PATH_INTERNAL        VARCHAR(400),
		  SET2_CREATE_IMGFOLDERS_WHERE  BIGINT,
		  SET2_IMG_FORMAT               VARCHAR(4),
		  SET2_IMG_THUMB_WIDTH          BIGINT,
		  SET2_IMG_THUMB_HEIGTH         BIGINT,
		  SET2_IMG_COMP_WIDTH           BIGINT,
		  SET2_IMG_COMP_HEIGTH          BIGINT,
		  SET2_IMG_DOWNLOAD_ORG         VARCHAR(2),
		  SET2_DOC_DOWNLOAD             VARCHAR(2),
		  SET2_INTRANET_REG_EMAILS      VARCHAR(1000),
		  SET2_INTRANET_REG_EMAILS_SUB  VARCHAR(500),
		  SET2_INTRANET_GEN_DOWNLOAD    VARCHAR(2),
		  SET2_CAT_WEB                  VARCHAR(2),
		  SET2_CAT_INTRA                VARCHAR(2),
		  SET2_URL_WEBSITE              VARCHAR(400),
		  SET2_PAYMENT_PRE              VARCHAR(2),
		  SET2_PAYMENT_BILL             VARCHAR(2),
		  SET2_PAYMENT_POD              VARCHAR(2),
		  SET2_PAYMENT_CC               VARCHAR(2),
		  SET2_PAYMENT_CC_CARDS         VARCHAR(500),
		  SET2_PAYMENT_PAYPAL           VARCHAR(2),
		  SET2_PATH_IMAGEMAGICK         VARCHAR(500),
		  SET2_EMAIL_SERVER             VARCHAR(200),
		  SET2_EMAIL_FROM               VARCHAR(200),
		  SET2_EMAIL_SMTP_USER          VARCHAR(200),
		  SET2_EMAIL_SMTP_PASSWORD      VARCHAR(200),
		  SET2_EMAIL_SERVER_PORT        BIGINT,
		  SET2_ORA_PATH_INCOMING		VARCHAR(500),
		  SET2_ORA_PATH_INCOMING_BATCH	VARCHAR(500),
		  SET2_ORA_PATH_OUTGOING		VARCHAR(500),
		  SET2_VID_PREVIEW_HEIGTH		BIGINT,
		  SET2_VID_PREVIEW_WIDTH		BIGINT,
		  SET2_PATH_FFMPEG				VARCHAR(500),
		  SET2_VID_PREVIEW_TIME			VARCHAR(10),
		  SET2_VID_PREVIEW_START		VARCHAR(10),
		  SET2_URL_SP_VIDEO				VARCHAR(500),
		  SET2_URL_SP_VIDEO_PREVIEW		VARCHAR(500),
		  SET2_VID_PREVIEW_AUTHOR		VARCHAR(200),
		  SET2_VID_PREVIEW_COPYRIGHT	VARCHAR(200),
		  SET2_CAT_VID_WEB				VARCHAR(2),
		  SET2_CAT_VID_INTRA			VARCHAR(2),
		  SET2_CAT_AUD_WEB				VARCHAR(2),
		  SET2_CAT_AUD_INTRA			VARCHAR(2),
		  SET2_CREATE_VIDFOLDERS_WHERE	BIGINT,
		  SET2_PATH_TO_ASSETS			VARCHAR(500),
		  SET2_PATH_TO_EXIFTOOL         VARCHAR(500),
		  SET2_NIRVANIX_NAME			VARCHAR(500),
		  SET2_NIRVANIX_PASS			VARCHAR(500),
		  HOST_ID						BIGINT,
		  SET2_AWS_BUCKET				VARCHAR(100),
		CONSTRAINT #arguments.thestruct.host_db_prefix#SETTINGS_2_FK FOREIGN KEY (HOST_ID)
		REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- KEYWORDS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#IMG_KEYWORDS
		(
		  KEY_ID     BIGINT NOT NULL,
		  KEY_LEVEL  BIGINT,
		  KEY_ID_R   BIGINT,
		  HOST_ID	 BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#IMG_KEYWORDS_PK PRIMARY KEY (KEY_ID),
		CONSTRAINT #arguments.thestruct.host_db_prefix#IMG_KEYWORDS_FK1 FOREIGN KEY (KEY_ID_R)
			  REFERENCES #arguments.thestruct.host_db_prefix#IMG_KEYWORDS (KEY_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- KEYWORDS TEXT --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#IMG_KEYWORDS_TEXT
		(
		  KEY_ID_R   BIGINT,
		  LANG_ID_R  BIGINT,
		  KEY_TEXT   VARCHAR(500),
		  HOST_ID	 BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#IMG_KEYWORDS_TEXT_FK1 FOREIGN KEY (KEY_ID_R)
			REFERENCES #arguments.thestruct.host_db_prefix#IMG_KEYWORDS (KEY_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- TEMP --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#TEMP
		(
		  TMP_TOKEN     VARCHAR(100),
		  TMP_FILENAME  VARCHAR(300),
		  HOST_ID		BIGINT
		)
		</cfquery>
		
		<!--- COLLECTIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#COLLECTIONS
		(
		  COL_ID        	BIGINT NOT NULL,
		  FOLDER_ID_R   	BIGINT DEFAULT NULL,
		  COL_OWNER     	BIGINT,
		  CREATE_DATE   	DATE,
		  CREATE_TIME   	TIMESTAMP,
		  CHANGE_DATE   	DATE,
		  CHANGE_TIME   	TIMESTAMP,
		  COL_TEMPLATE  	VARCHAR(100),
		  COL_SHARED		VARCHAR(2) DEFAULT 'F',
		  COL_NAME_SHARED	VARCHAR(200),
		  share_dl_org		varchar(1) DEFAULT 'f',
     	  share_comments	varchar(1) DEFAULT 'f',
		  share_upload		varchar(1) DEFAULT 'f',
		  share_order		varchar(1) DEFAULT 'f',
		  share_order_user	BIGINT,
		  HOST_ID			BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#COLLECTIONS_PK PRIMARY KEY (COL_ID),
		CONSTRAINT #arguments.thestruct.host_db_prefix#COLLECTIONS_#arguments.thestruct.host_db_prefix#COL_FK1_HOSTS FOREIGN KEY (HOST_ID)
		REFERENCES HOSTS (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- COLLECTIONS TEXT --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#COLLECTIONS_TEXT
		(
		  COL_ID_R      BIGINT,
		  LANG_ID_R     BIGINT,
		  COL_DESC      VARCHAR(1000),
		  COL_KEYWORDS  VARCHAR(2000),
		  COL_NAME      VARCHAR(300),
		  HOST_ID		BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#COLLECTIONS_TEXT_#arguments.thestruct.host_db_prefix#FK1 FOREIGN KEY (COL_ID_R)
			REFERENCES #arguments.thestruct.host_db_prefix#COLLECTIONS (COL_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- COLLECTIONS FILES CROSS TABLE --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#COLLECTIONS_CT_FILES
		(
		  COL_ID_R       	BIGINT,
		  FILE_ID_R      	BIGINT,
		  COL_FILE_TYPE  	VARCHAR(5),
		  COL_ITEM_ORDER  	BIGINT,
		  COL_FILE_FORMAT  	VARCHAR(20),
		  HOST_ID			BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#COLLECTIONS_CT_FILES_FK1 FOREIGN KEY (COL_ID_R)
			REFERENCES #arguments.thestruct.host_db_prefix#COLLECTIONS (COL_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- COLLECTIONS GROUPS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#COLLECTIONS_GROUPS
		(
		  COL_ID_R       	BIGINT,
		  GRP_ID_R			BIGINT DEFAULT NULL,
		  GRP_PERMISSION	VARCHAR(2),
		  HOST_ID			BIGINT,
		  CONSTRAINT #arguments.thestruct.host_db_prefix#COLLECTIONS_GROUPS_FK1 FOREIGN KEY (col_id_r)
		  REFERENCES #arguments.thestruct.host_db_prefix#COLLECTIONS (col_id) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- USER FAVORITES --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#USERS_FAVORITES
		(
		  USER_ID_R  BIGINT,
		  FAV_TYPE   VARCHAR(8),
		  FAV_ID     BIGINT,
		  FAV_KIND   VARCHAR(8),
		  FAV_ORDER  BIGINT,
		  HOST_ID	 BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#USERS_FAVORITES_FK1 FOREIGN KEY (USER_ID_R)
			  REFERENCES USERS (USER_ID) ON DELETE SET NULL
		)
		</cfquery>
		
		<!--- VIDEOS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#VIDEOS
		(
		VID_ID					BIGINT,
		VID_FILENAME			VARCHAR(200),
		FOLDER_ID_R				BIGINT DEFAULT NULL,
		VID_CUSTOM_ID			VARCHAR(100),
		VID_ONLINE				VARCHAR(2),
		VID_OWNER				BIGINT,
		VID_CREATE_DATE			DATE,
		VID_CREATE_TIME			TIMESTAMP,
		VID_CHANGE_DATE			DATE,
		VID_CHANGE_TIME			TIMESTAMP,
		VID_RANKING				BIGINT,
		VID_SINGLE_SALE			VARCHAR(2),
		VID_IS_NEW				VARCHAR(2),
		VID_SELECTION			VARCHAR(2),
		VID_IN_PROGRESS			VARCHAR(2),
		VID_LICENSE				VARCHAR(200),
		VID_CATEGORY_ONE		VARCHAR(2000),
		VID_REMARKS				VARCHAR(1),
		VID_WIDTH				BIGINT,
		VID_HEIGHT				BIGINT,
		VID_FRAMERESOLUTION		BIGINT,
		VID_FRAMERATE			BIGINT,
		VID_VIDEODURATION		BIGINT,
		VID_COMPRESSIONTYPE		VARCHAR(4000),
		VID_BITRATE				BIGINT,
		VID_EXTENSION			VARCHAR(10),
		VID_MIMETYPE			VARCHAR(500),
		VID_PREVIEW_WIDTH		BIGINT,
		VID_PREVIEW_HEIGTH		BIGINT,
		VID_GROUP				BIGINT DEFAULT NULL,
		VID_PUBLISHER			VARCHAR(200),
		VID_NAME_ORG			VARCHAR(200),
		VID_NAME_IMAGE			VARCHAR(200),
		VID_NAME_PRE			VARCHAR(200),
		VID_NAME_PRE_IMG		VARCHAR(200),
	 	VID_SIZE                BIGINT,
	 	VID_PREV_SIZE           BIGINT,
	 	LUCENE_KEY		   		VARCHAR(2000),
	 	SHARED			 		VARCHAR(2) DEFAULT 'F',
	 	LINK_KIND			    VARCHAR(20),
		LINK_PATH_URL		    VARCHAR(2000),
		VID_META				CLOB,
		HOST_ID					BIGINT,
		PATH_TO_ASSET		  	VARCHAR(100),
		CLOUD_URL		   	    VARCHAR(500),
		CLOUD_URL_ORG		    VARCHAR(500),
		HASHTAG			   		VARCHAR(100),
		IS_AVAILABLE		  VARCHAR(1) DEFAULT 0,
		CLOUD_URL_EXP		   BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#VIDEO_PK PRIMARY KEY (VID_ID),
		CONSTRAINT #arguments.thestruct.host_db_prefix#VIDEO_FK1 FOREIGN KEY (HOST_ID)
		REFERENCES hosts (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- VIDEOS TEXT --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#VIDEOS_TEXT
		(
		  VID_ID_R         BIGINT NOT NULL,
		  LANG_ID_R        BIGINT NOT NULL,
		  VID_KEYWORDS     VARCHAR(4000),
		  VID_DESCRIPTION  VARCHAR(4000),
		  VID_TITLE		   VARCHAR(400),
		  ID			   BIGINT AUTO_INCREMENT,
		  HOST_ID		   BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#VIDEO_TEXT_FK_VID FOREIGN KEY (VID_ID_R)
			REFERENCES #arguments.thestruct.host_db_prefix#VIDEOS (VID_ID)
			ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- SCHEDULES --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#SCHEDULES
		(
			SCHED_ID 			 BIGINT NOT NULL,
			SET2_ID_R 			 BIGINT,
			SCHED_USER 			 BIGINT,
			SCHED_STATUS 		 VARCHAR(1) DEFAULT 1,
			SCHED_METHOD 		 VARCHAR(10),
			SCHED_NAME 			 VARCHAR(255),
			SCHED_FOLDER_ID_R    BIGINT,
			SCHED_ZIP_EXTRACT 	 BIGINT,
			SCHED_SERVER_FOLDER  VARCHAR(4000),
			SCHED_SERVER_RECURSE BIGINT DEFAULT 1,
			SCHED_SERVER_FILES   BIGINT DEFAULT 0,
			SCHED_MAIL_POP 		 VARCHAR(255),
			SCHED_MAIL_USER 	 VARCHAR(255),
			SCHED_MAIL_PASS 	 VARCHAR(255),
			SCHED_MAIL_SUBJECT 	 VARCHAR(255),
			SCHED_FTP_SERVER 	 VARCHAR(255),
			SCHED_FTP_USER 		 VARCHAR(255),
			SCHED_FTP_PASS 		 VARCHAR(255),
			SCHED_FTP_PASSIVE    BIGINT DEFAULT 0,
			SCHED_FTP_FOLDER 	 VARCHAR(255),
			SCHED_INTERVAL       VARCHAR(255),
			SCHED_START_DATE     DATE,
			SCHED_START_TIME     TIMESTAMP,
			SCHED_END_DATE       DATE,
			SCHED_END_TIME       TIMESTAMP,
			HOST_ID				 BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#SCHEDULES_PK PRIMARY KEY (SCHED_ID),
		CONSTRAINT #arguments.thestruct.host_db_prefix#SCHEDULES_FK1 FOREIGN KEY (HOST_ID)
		REFERENCES hosts (HOST_ID) ON DELETE CASCADE,
		CONSTRAINT #arguments.thestruct.host_db_prefix#SCHEDULES_FK2 FOREIGN KEY (SCHED_USER)
		REFERENCES USERS (USER_ID) ON DELETE SET NULL
		)
		</cfquery>
		
		<!--- SCHEDULES_LOG --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#SCHEDULES_LOG
		(
			SCHED_LOG_ID        BIGINT NOT NULL,
			SCHED_ID_R          BIGINT,
			SCHED_LOG_USER      BIGINT,
			SCHED_LOG_ACTION    VARCHAR(10),
			SCHED_LOG_DATE      DATE,
			SCHED_LOG_TIME      TIMESTAMP,
			SCHED_LOG_DESC      VARCHAR(4000),
			HOST_ID				BIGINT,
		CONSTRAINT #arguments.thestruct.host_db_prefix#SCHEDULES_LOG_PK PRIMARY KEY (SCHED_LOG_ID),
		CONSTRAINT #arguments.thestruct.host_db_prefix#SCHEDULES_LOG_FK1 FOREIGN KEY (SCHED_ID_R)
		REFERENCES #arguments.thestruct.host_db_prefix#SCHEDULES (SCHED_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- CUSTOM FIELDS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#custom_fields
		(
			cf_id 			bigint, 
			cf_type	 		VARCHAR(20), 
			cf_order 		bigint, 
			cf_enabled 		VARCHAR(2), 
			cf_show			VARCHAR(10),
			cf_group 		VARCHAR(100),
			host_id			BIGINT
		)
		</cfquery>
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#custom_fields_text
		(
			cf_id_r			bigint, 
			lang_id_r 		bigint, 
			cf_text			VARCHAR(500),
			HOST_ID			BIGINT,
			CONSTRAINT #arguments.thestruct.host_db_prefix#cf_text FOREIGN KEY (cf_id_r) REFERENCES #arguments.thestruct.host_db_prefix#custom_fields (cf_id) ON DELETE CASCADE
		)
		</cfquery>
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#custom_fields_values
		(
			cf_id_r			bigint, 
			asset_id_r 		bigint, 
			cf_value		VARCHAR(4000),
			HOST_ID			BIGINT,
			CONSTRAINT #arguments.thestruct.host_db_prefix#cf_values FOREIGN KEY (cf_id_r) REFERENCES #arguments.thestruct.host_db_prefix#custom_fields (cf_id) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- COMMENTS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#comments
		(
			COM_ID			VARCHAR(100),
			ASSET_ID_R		BIGINT,
			ASSET_TYPE		VARCHAR(10),
			USER_ID_R		BIGINT,
			COM_TEXT		VARCHAR(4000),
			COM_DATE		TIMESTAMP,
			HOST_ID			BIGINT,
			CONSTRAINT #arguments.thestruct.host_db_prefix#comments_pk PRIMARY KEY (COM_ID)
		)
		</cfquery>
		
		<!--- Versions --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#versions
		(
			asset_id_r			BIGINT,
			ver_version			BIGINT DEFAULT NULL,
			ver_type			VARCHAR(5),
			ver_date_add		TIMESTAMP,
			ver_who				BIGINT,
			ver_filename_org 	VARCHAR(200),
			ver_extension	 	VARCHAR(7),
			thumb_width			BIGINT,
			thumb_height		BIGINT,
			img_width			BIGINT,
			img_height			BIGINT,
			img_size			BIGINT,
			thumb_size			BIGINT,
			vid_size			BIGINT,
			vid_width			BIGINT,
			vid_height			BIGINT,
			vid_name_image		VARCHAR(200),
			HOST_ID				BIGINT,
			cloud_url_org		VARCHAR(500),
			ver_thumbnail		VARCHAR(200)
		)
		</cfquery>
		
		<!--- TRANSLATIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#languages
		(
			lang_id			BIGINT NOT NULL,
			lang_name		VARCHAR(100),
			lang_active		VARCHAR(2) default 'f',
			host_id			BIGINT
		)
		</cfquery>
		
		<!--- AUDIOS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#audios
		(
			aud_ID              BIGINT,
			FOLDER_ID_R         BIGINT DEFAULT NULL,
			aud_CREATE_DATE     DATE,
			aud_CREATE_TIME     TIMESTAMP,
			aud_CHANGE_DATE     DATE,
			aud_CHANGE_TIME     TIMESTAMP,
			aud_OWNER           BIGINT,
			aud_TYPE            VARCHAR(5),
			aud_NAME            VARCHAR(200),
			aud_EXTENSION       VARCHAR(6),
			aud_NAME_NOEXT      VARCHAR(200),
			aud_CONTENTTYPE     VARCHAR(100),
			aud_CONTENTSUBTYPE  VARCHAR(100),
			aud_ONLINE          VARCHAR(2),
			aud_NAME_ORG        VARCHAR(200),
			aud_GROUP           BIGINT DEFAULT NULL,
			aud_size			BIGINT,
			LUCENE_KEY		   	VARCHAR(2000),
			SHARED			   	VARCHAR(2) DEFAULT 'F',
			aud_meta			CLOB,
			LINK_KIND			VARCHAR(20),
		 	LINK_PATH_URL		VARCHAR(2000),
		 	HOST_ID				BIGINT,
		 	PATH_TO_ASSET		VARCHAR(100),
		 	CLOUD_URL			VARCHAR(500),
		 	CLOUD_URL_2		    VARCHAR(500),
		    CLOUD_URL_ORG		VARCHAR(500),
		    HASHTAG			    VARCHAR(100),
		    IS_AVAILABLE		  VARCHAR(1) DEFAULT 0,
		    CLOUD_URL_EXP		   BIGINT,
			CONSTRAINT #arguments.thestruct.host_db_prefix#audios_PK PRIMARY KEY (aud_ID),
			CONSTRAINT #arguments.thestruct.host_db_prefix#audios_FK1 FOREIGN KEY (HOST_ID) REFERENCES hosts (HOST_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- AUDIOS TEXT --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#audios_text
		(
			aud_ID_R			BIGINT,
			LANG_ID_R			BIGINT,
			aud_DESCRIPTION     VARCHAR(4000),
			aud_KEYWORDS		VARCHAR(4000),
			ID			   		BIGINT AUTO_INCREMENT,
			HOST_ID				BIGINT,
			CONSTRAINT #arguments.thestruct.host_db_prefix#audios_DESC_FK_FILE FOREIGN KEY (aud_ID_R)
			REFERENCES #arguments.thestruct.host_db_prefix#audios (aud_ID) ON DELETE CASCADE
		)
		</cfquery>
		
		<!--- SHARE OPTIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE TABLE #arguments.thestruct.host_db_prefix#share_options
		(
			asset_id_r		BIGINT,
			host_id			BIGINT,
			group_asset_id	BIGINT,
			folder_id_r		BIGINT,
			asset_type		varchar(6),
			asset_format	varchar(10),
			asset_dl		varchar(1) DEFAULT '0',
			asset_order		varchar(1) DEFAULT '0'
		)
		</cfquery>
				
	</cffunction>
	
	<!--- Create Indexes --->
	<cffunction name="create_indexes" access="public" output="false">
		<cfargument name="thestruct" type="Struct">
		<!--- FILES --->
		<!--- <cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#DOC_CONTENT_IDX ON #arguments.thestruct.host_db_prefix#FILES
		(FILE_BINARY)
		</cfquery> --->
		<!--- FILES DESC --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#DOC_DESCRIPTION_IDX ON #arguments.thestruct.host_db_prefix#FILES_DESC
		(FILE_DESC)
		</cfquery>
		<!--- FILES KEYWORDS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#DOC_KEYWORDS_IDX ON #arguments.thestruct.host_db_prefix#FILES_DESC
		(FILE_KEYWORDS)
		</cfquery>
		<!--- IMAGE DESCRIPTIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#IMG_DESCRIPTION_IDX ON #arguments.thestruct.host_db_prefix#IMAGES_TEXT
		(IMG_DESCRIPTION)
		</cfquery>
		<!--- IMAGE KEYWORDS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#IMG_KEYWORDS_IDX ON #arguments.thestruct.host_db_prefix#IMAGES_TEXT
		(IMG_KEYWORDS)
		</cfquery>
		<!--- IMAGE XMP --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#IMG_XMP_IDX ON #arguments.thestruct.host_db_prefix#IMAGES
		(metaxmp)
		</cfquery>
		<!--- VIDEO DESCRIPTIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#VID_DESCRIPTION_IDX ON #arguments.thestruct.host_db_prefix#VIDEOS_TEXT
		(VID_DESCRIPTION)
		</cfquery>
		<!--- VIDEO KEYWORDS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#VID_KEYWORDS_IDX ON #arguments.thestruct.host_db_prefix#VIDEOS_TEXT
		(VID_KEYWORDS)
		</cfquery>
		<!--- AUDIO DESCRIPTIONS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#AUD_DESCRIPTION_IDX ON #arguments.thestruct.host_db_prefix#AUDIOS_TEXT
		(AUD_DESCRIPTION)
		</cfquery>
		<!--- AUDIO KEYWORDS --->
		<cfquery datasource="#arguments.thestruct.dsn#">
		CREATE INDEX #arguments.thestruct.host_db_prefix#AUD_KEYWORDS_IDX ON #arguments.thestruct.host_db_prefix#AUDIOS_TEXT
		(AUD_KEYWORDS)
		</cfquery>
		<cfreturn />
	</cffunction>
	
	<!--- Clear database completely --->
	<cffunction name="clearall" access="public" output="false">
		<!--- Query Tables --->
		<cfquery datasource="#session.firsttime.database#" name="qrytables">
		SELECT table_name
		FROM information_schema.tables
		WHERE table_catalog = 'RAZUNA'
		AND table_type = 'TABLE'
		ORDER BY table_name
		</cfquery>
		<!--- Loop and drop tables --->
		<cfloop query="qrytables">
			<cfquery datasource="#session.firsttime.database#">
			ALTER TABLE #table_name# SET REFERENTIAL_INTEGRITY false
			</cfquery>
			<cfquery datasource="#session.firsttime.database#">
			DROP TABLE #table_name#
			</cfquery>
		</cfloop>
		<!--- Query Sequences --->
		<cfquery datasource="#session.firsttime.database#" name="qryseq">
		SELECT sequence_name as theseq
		FROM information_schema.sequences
		WHERE IS_GENERATED = false
		</cfquery>
		<!--- Loop over Sequences and remove them --->
		<cfloop query="qryseq">
			<cfquery datasource="#session.firsttime.database#">
			DROP SEQUENCE #theseq#
			</cfquery>
		</cfloop>
		<cfreturn />
	</cffunction>
	
	<!--- OPENBD CONFIG INTERACTION --->
	
	<cffunction name="bdgetConfig" access="private" output="false" returntype="struct" hint="Returns a struct representation of the OpenBD server configuration (bluedragon.xml)">
		<cfset var admin = "" />
			<cflock scope="Server" type="readonly" timeout="5">
				<cfset admin = createObject("java", "com.naryx.tagfusion.cfm.engine.cfEngine").getConfig().getCFMLData() />
			</cflock>
		<cfreturn admin.server />
	</cffunction>
	
	<cffunction name="bdsetConfig" access="private" output="false" returntype="void" hint="Sets the server configuration and tells OpenBD to refresh its settings">
		<cfargument name="currentConfig" type="struct" required="true" hint="The configuration struct, which is a struct representation of bluedragon.xml" />
			<cflock scope="Server" type="exclusive" timeout="5">
				<cfset admin.server = duplicate(arguments.currentConfig) />
				<cfset admin.server.openbdadminapi.lastupdated = DateFormat(now(), "dd/mmm/yyyy") & " " & TimeFormat(now(), "HH:mm:ss") />
				<cfset admin.server.openbdadminapi.version = "1.0" />
				<cfset xmlConfig = createObject("java", "com.naryx.tagfusion.xmlConfig.xmlCFML").init(admin) />
				<cfset success = createObject("java", "com.naryx.tagfusion.cfm.engine.cfEngine").writeXmlFile(xmlConfig) />
			</cflock>
	</cffunction>
	
	<cffunction name="bddatasourceExists" access="public" output="false" returntype="boolean" hint="Returns a boolean indicating whether or not a datasource with the specified name exists">
		<cfargument name="dsn" type="string" required="true" hint="The datasource name to check" />
		<cfset var dsnExists = true />
		<cfset var localConfig = bdgetConfig() />
		<cfset var i = 0 />
		<cfif not StructKeyExists(localConfig, "cfquery") or not StructKeyExists(localConfig.cfquery, "datasource")>
			<!--- no datasources at all, so this one doesn't exist ---->
			<cfset dsnExists = false />
		<cfelse>
			<cfloop index="i" from="1" to="#ArrayLen(localConfig.cfquery.datasource)#">
				<cfif localConfig.cfquery.datasource[i].name is arguments.dsn>
					<cfset dsnExists = true />
					<cfbreak />
				<cfelse>
					<cfset dsnExists = false />
				</cfif>
			</cfloop>
		</cfif>
		<cfreturn dsnExists />
	</cffunction>
	
	<cffunction name="BDsetDatasource" access="public" output="false" returntype="void" hint="Creates or updates a datasource">
		<cfargument name="name" type="string" required="true" hint="OpenBD Datasource Name" />
		<cfargument name="databasename" type="string" required="false" default="" hint="Database name on the database server" />
		<cfargument name="server" type="string" required="false" default="" hint="Database server host name or IP address" />
		<cfargument name="port"	type="numeric" required="false" default="0" hint="Port that is used to access the database server" />
		<cfargument name="username" type="string" required="false" default="" hint="Database username" />
		<cfargument name="password" type="string" required="false" default="" hint="Database password" />
		<cfargument name="hoststring" type="string" required="false" default="" hint="JDBC URL for 'other' database types. Databasename, server, and port arguments are ignored if a hoststring is provided." />
		<cfargument name="description" type="string" required="false" default="" hint="A description of this data source" />
		<cfargument name="initstring" type="string" required="false" default="" hint="Additional initialization settings" />
		<cfargument name="connectiontimeout" type="numeric" required="false" default="120" hint="Number of seconds OpenBD maintains an unused connection before it is destroyed" />
		<cfargument name="connectionretries" type="numeric" required="false" default="0" hint="Number of connection retry attempts to make" />
		<cfargument name="logintimeout" type="numeric" required="false" default="120" hint="Number of seconds before OpenBD times out the data source connection login attempt" />
		<cfargument name="maxconnections" type="numeric" required="false" default="3" hint="Maximum number of simultaneous database connections" />
		<cfargument name="perrequestconnections" type="boolean" required="false" default="false" hint="Indication of whether or not to pool connections" />
		<cfargument name="sqlselect" type="boolean" required="false" default="true" hint="Allow SQL SELECT statements from this datasource" />
		<cfargument name="sqlinsert" type="boolean" required="false" default="true" hint="Allow SQL INSERT statements from this datasource" />
		<cfargument name="sqlupdate" type="boolean" required="false" default="true" hint="Allow SQL UPDATE statements from this datasource" />
		<cfargument name="sqldelete" type="boolean" required="false" default="true" hint="Allow SQL DELETE statements from this datasource" />
		<cfargument name="sqlstoredprocedures" type="boolean" required="false" default="true" hint="Allow SQL stored procedure calls from this datasource" />
		<cfargument name="drivername" type="string" required="false" default="" hint="JDBC driver class to use" />
		<cfargument name="action" type="string" required="false" default="create" hint="Action to take on the datasource (create or update)" />
		<cfargument name="existingDatasourceName" type="string" required="false" default="" hint="The existing (old) datasource name so we know what to delete if this is an update" />
		<cfargument name="verificationQuery" type="string" required="false" default="" hint="Custom verification query for 'other' driver types" />
		
		<cfset var localConfig = bdgetConfig() />
		<cfset var datasourceSettings = structNew() />
		
		<!--- make sure configuration structure exists, otherwise build it --->
		<cfif (NOT StructKeyExists(localConfig, "cfquery")) OR (NOT StructKeyExists(localConfig.cfquery, "datasource"))>
			<cfset localConfig.cfquery.datasource = ArrayNew(1) />
		</cfif>
		
		<!--- if the datasource already exists and this isn't an update, throw an error --->
		<cfif bddatasourceExists(arguments.name) EQ "false">
			<!--- build up the universal datasource settings --->
			<cfscript>
				// Set the params
				datasourceSettings.name = trim(lcase(arguments.name));
				datasourceSettings.displayname = arguments.name;
				datasourceSettings.databasename = trim(arguments.databasename);
				datasourceSettings.username = trim(arguments.username);
				datasourceSettings.password = trim(arguments.password);
				datasourceSettings.drivername = trim(arguments.drivername);
				datasourceSettings.initstring = trim(arguments.initstring);
				datasourceSettings.sqlselect = ToString(arguments.sqlselect);
				datasourceSettings.sqlinsert = ToString(arguments.sqlinsert);
				datasourceSettings.sqlupdate = ToString(arguments.sqlupdate);
				datasourceSettings.sqldelete = ToString(arguments.sqldelete);
				datasourceSettings.sqlstoredprocedures = ToString(arguments.sqlstoredprocedures);
				datasourceSettings.logintimeout = ToString(arguments.logintimeout);
				datasourceSettings.connectiontimeout = ToString(arguments.connectiontimeout);
				datasourceSettings.connectionretries = ToString(arguments.connectionretries);
				datasourceSettings.maxconnections = ToString(arguments.maxconnections);
				datasourceSettings.perrequestconnections = ToString(arguments.perrequestconnections);
				datasourceSettings.hoststring = ToString(arguments.hoststring);
				// prepend the new datasource to the localconfig array
				arrayPrepend(localConfig.cfquery.datasource, structCopy(datasourceSettings));
				// update the config
				bdsetConfig(localConfig);
			</cfscript>
		</cfif>
	</cffunction>
		
</cfcomponent>