<!---
*
* Copyright (C) 2005-2008 Razuna Ltd.
*
* This file is part of Razuna - Enterprise Digital Asset Management.
*
* Razuna is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Razuna is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero Public License for more details.
*
* You should have received a copy of the GNU Affero Public License
* along with Razuna. If not, see <http://www.gnu.org/licenses/>.
*
* You may restribute this Program with a special exception to the terms
* and conditions of version 3.0 of the AGPL as described in Razuna's
* FLOSS exception. You should have received a copy of the FLOSS exception
* along with Razuna. If not, see <http://www.razuna.com/licenses/>.
*
--->

<!---  --->
<!--- UPDATE FROM 1.3.5 TO 1.4 --->
<!---  --->

<cfcomponent output="false">
	
	<!--- DO DB update --->
	<cffunction name="update_do">
		<!--- Param --->
		<cfset arguments.thestruct = structnew()>
		<!--- Name for the log --->
		<cfset logname = "razuna_update_" & dateformat(now(),"mm_dd_yyyy") & "_" & timeformat(now(),"HH-mm-ss")>
		<!--- ORACLE / H2 --->
		<cfif application.razuna.thedatabase EQ "oracle" OR application.razuna.thedatabase EQ "h2">
			<!--- Core DB --->
			<cftry>
				<cfquery datasource="#application.razuna.datasource#">
				CREATE TABLE search_reindex
				(
					theid		VARCHAR2(100 CHAR),
					thevalue	NUMBER,
					thehostid 	NUMBER,
					datetime 	TIMESTAMP
				)
				</cfquery>
				<cfcatch type="any">
					<cflog application="no" file="#logname#" type="error" text="Table search_reindex: message: #cfcatch.message# Detail: #cfcatch.detail#">
				</cfcatch>
			</cftry>
			<cftry>
				<cfquery datasource="#application.razuna.datasource#">
				ALTER TABLE hosts ADD host_shard_group varchar2(10 CHAR)
				</cfquery>
				<cfcatch type="any">
					<cflog application="no" file="#logname#" type="error" text="Table host_shard_group: message: #cfcatch.message# Detail: #cfcatch.detail#">
				</cfcatch>
			</cftry>
		<!--- MYSQL --->
		<cfelseif application.razuna.thedatabase EQ "mysql">
			<!--- Core DB --->
			<cftry>
				<cfquery datasource="#application.razuna.datasource#">
				CREATE TABLE search_reindex
				(
					theid 		VARCHAR(100),
					thevalue 	INT,
					thehostid 	INT,
					datetime 	TIMESTAMP
				)
				#this.tableoptions#
				</cfquery>
				<cfcatch type="any">
					<cflog application="no" file="#logname#" type="error" text="Table search_reindex: message: #cfcatch.message# Detail: #cfcatch.detail#">
				</cfcatch>
			</cftry>
			<cftry>
				<cfquery datasource="#application.razuna.datasource#">
				ALTER TABLE hosts ADD COLUMN host_shard_group varchar(10)
				</cfquery>
				<cfcatch type="any">
					<cflog application="no" file="#logname#" type="error" text="Table search_reindex: message: #cfcatch.message# Detail: #cfcatch.detail#">
				</cfcatch>
			</cftry>
		<!--- MSSQL --->
		<cfelseif application.razuna.thedatabase EQ "mssql">
			<!--- Core DB --->
			<cftry>
				<cfquery datasource="#application.razuna.datasource#">
				CREATE TABLE search_reindex
				(
					theid 		VARCHAR(100),
					thevalue 	INT,
					thehostid 	INT,
					datetime 	DATETIME
				)
				</cfquery>
				<cfcatch type="any">
					<cflog application="no" file="#logname#" type="error" text="Table search_reindex: message: #cfcatch.message# Detail: #cfcatch.detail#">
				</cfcatch>
			</cftry>
			<cftry>
				<cfquery datasource="#application.razuna.datasource#">
				ALTER TABLE hosts ADD host_shard_group varchar(10)
				</cfquery>
				<cfcatch type="any">
					<cflog application="no" file="#logname#" type="error" text="Table search_reindex: message: #cfcatch.message# Detail: #cfcatch.detail#">
				</cfcatch>
			</cftry>
		</cfif>
		<!--- Create the RAZ1_ tables --->
		<cfset arguments.thestruct.dsn = application.razuna.datasource>
		<cfset arguments.thestruct.host_db_prefix = "raz1_">
		<cfset arguments.thestruct.theschema = application.razuna.theschema>
		<cfinvoke component="db_#application.razuna.thedatabase#" method="create_host" thestruct="#arguments.thestruct#">
		<!--- Set sharding group to raz1_ for all hosts --->
		<cfquery datasource="#application.razuna.datasource#">
		UPDATE hosts
		SET host_shard_group = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.thestruct.host_db_prefix#">
		</cfquery>
		<!--- Get all hosts --->
		<cfquery datasource="#application.razuna.datasource#" name="hostslist">
		SELECT host_id, host_name, host_path, host_create_date, host_db_prefix, host_lang
		FROM hosts
		ORDER BY lower(host_name)
		</cfquery>
		<!--- Loop over hosts, get tables and insert into raz1_ tables --->
		<cfloop query="hostslist">
			<!--- Put host_db_prefix & host_id in variable since the loop below does not see it --->
			<cfset theprefix = host_db_prefix>
			<cfset thehostid = host_id>
			<!--- Oracle --->
			<cfif application.razuna.thedatabase EQ "oracle">
				<cfquery datasource="#application.razuna.datasource#" name="qryt">
				SELECT object_name as thetable
				FROM user_objects 
				WHERE object_type='TABLE' 
				AND object_name LIKE <cfqueryparam cfsqltype="cf_sql_varchar" value="#ucase(theprefix)#%">
				</cfquery>
			<!--- all other DBs --->
			<cfelse>
				<cfquery datasource="#application.razuna.datasource#" name="qryt">
				SELECT table_name as thetable
				FROM information_schema.tables
				WHERE table_name LIKE <cfqueryparam cfsqltype="cf_sql_varchar" value="#ucase(theprefix)#%">
				</cfquery>
			</cfif>
			<!--- Now insert with select --->
			<cfloop query="qryt">
				<cftry>
					<!--- Set the raz1_ db name --->
					<cfset raz_table = replacenocase(thetable,theprefix,arguments.thestruct.host_db_prefix,"one")>
					<!--- Get Columns --->
					<cfquery datasource="#application.razuna.datasource#" name="qry_columns">
					SELECT column_name
					FROM information_schema.columns
					WHERE table_name = <cfqueryparam cfsqltype="cf_sql_varchar" value="#ucase(thetable)#">
					AND column_name != 'ADMIN'
					AND column_name != 'NAME'
					AND column_name != 'REMARKS'
					AND column_name != 'ID'
					AND column_name != 'IMAGE'
					AND column_name != 'THUMB'
					AND column_name != 'COMP'
					AND column_name != 'COMP_UW'
					AND column_name != 'SET2_INTRANET_LOGO'
					</cfquery>
					<!--- Set column list --->
					<cfset thecolumns = valuelist(qry_columns.column_name)>
					<cfquery datasource="#application.razuna.datasource#">
					INSERT INTO #raz_table#
					(#thecolumns#)
					SELECT #thecolumns#
					FROM #thetable#
					</cfquery>
					<!--- And add the host_id to this table --->
					<cfquery datasource="#application.razuna.datasource#">
					UPDATE #raz_table#
					SET host_id = <cfqueryparam cfsqltype="cf_sql_numeric" value="#thehostid#">
					</cfquery>
					<cfcatch type="any">
						<cflog application="no" file="#logname#" type="error" text="Table #thetable#: message: #cfcatch.message# Detail: #cfcatch.detail#">
					</cfcatch>
				</cftry>
			</cfloop>
		</cfloop>
		<!--- Now set the path_to_asset --->
		<cfquery dataSource="#application.razuna.datasource#" name="x">
		SELECT img_id, folder_id_r
		FROM raz1_images
		</cfquery>
		<cfloop query="x">
			<cfquery dataSource="#application.razuna.datasource#">
			update raz1_images
			set path_to_asset = '#folder_id_r#/img/#img_id#',
			is_available = '1'
			where img_id = #img_id#
			</cfquery>
		</cfloop>
		<cfquery dataSource="#application.razuna.datasource#" name="x">
		SELECT file_id, folder_id_r
		FROM raz1_files
		</cfquery>
		<cfloop query="x">
			<cfquery dataSource="#application.razuna.datasource#">
			update raz1_files
			set path_to_asset = '#folder_id_r#/doc/#file_id#',
			is_available = '1'
			where file_id = #file_id#
			</cfquery>
		</cfloop>
		<cfquery dataSource="#application.razuna.datasource#" name="x">
		SELECT vid_id, folder_id_r
		FROM raz1_videos
		</cfquery>
		<cfloop query="x">
			<cfquery dataSource="#application.razuna.datasource#">
			update raz1_videos
			set path_to_asset = '#folder_id_r#/vid/#vid_id#',
			is_available = '1'
			where vid_id = #vid_id#
			</cfquery>
		</cfloop>
		<cfquery dataSource="#application.razuna.datasource#" name="x">
		SELECT aud_id, folder_id_r
		FROM raz1_audios
		</cfquery>
		<cfloop query="x">
			<cfquery dataSource="#application.razuna.datasource#">
			update raz1_audios
			set path_to_asset = '#folder_id_r#/aud/#aud_id#',
			is_available = '1'
			where aud_id = #aud_id#
			</cfquery>
		</cfloop>
		<!--- Set new file types --->
		<cftry>
			<cfquery dataSource="#application.razuna.datasource#">
			INSERT INTO file_types VALUES ('ogv', 'vid', 'video', 'ogv')
			</cfquery>
			<cfcatch type="any"></cfcatch>
		</cftry>
		<cftry>
			<cfquery dataSource="#application.razuna.datasource#">
			INSERT INTO file_types VALUES ('webm', 'vid', 'video', 'webm')
			</cfquery>
			<cfcatch type="any"></cfcatch>
		</cftry>
		<!--- All done and update should be flying --->
	</cffunction>



</cfcomponent>